import Visitor

class ASTPrinter(Visitor.Visitor):
    """
    Visitor for printing an AST for debugging purposes.
    """
    
    def __init__(self, indent_amount=4):
        self.level = 0
        self.indent_amount = indent_amount

    def printline(self, line):
        print '%s%s' % (' '*self.level, line)

    def _inc_level(self):
        self.level += self.indent_amount

    def _dec_level(self):
        self.level -= self.indent_amount

    def visit_BlockRule(self, astnode, arg):
        self.printline(astnode)
        self._inc_level()
        for rule in astnode.rules:
            rule.visit(self)
        self._dec_level()

    def visit_IfBlockRule(self, astnode, arg):
        self.printline(astnode)
        self._inc_level()
        for rule in astnode.rules:
            rule.visit(self)
        astnode.if_expr.visit(self)
        self._dec_level()

    def visit_GloblistRule(self, astnode, arg):
        self.printline(astnode)
        self._inc_level()
        for pattern in astnode.glob_patterns:
            pattern.visit(self)
            assert pattern.parent == astnode
        if astnode.if_expr:
            self.printline('If-expression:')
            astnode.if_expr.visit(self)
        self._dec_level()

    def visit_ForEachRule(self, astnode, arg):
        self.printline(astnode)
        self._inc_level()
        for pattern in astnode.glob_patterns:
            pattern.visit(self)
            assert pattern.parent == astnode
#        self.printline('If-expression:')
        astnode.if_expr.visit(self)
        self._dec_level()

    def visit_GlobPattern(self, astnode, arg):
        self.printline(str(astnode)) # + ' (innerdir "%s")' % astnode.innerdir)

    def visit_AndExpression(self, astnode, arg):
        self.printline(astnode)
        self._inc_level()
        astnode.expr1.visit(self)
        astnode.expr2.visit(self)
        assert astnode.expr1.parent == astnode
        assert astnode.expr2.parent == astnode
        self._dec_level()

    def visit_OrExpression(self, astnode, arg):
        self.printline(astnode)
        self._inc_level()
        astnode.expr1.visit(self)
        astnode.expr2.visit(self)
        assert astnode.expr1.parent == astnode
        assert astnode.expr2.parent == astnode
        self._dec_level()

    def visit_NotExpression(self, astnode, arg):
        self.printline(astnode)
        self._inc_level()
        astnode.expr.visit(self)
        self._dec_level()

    def visit_FunctionExpression(self, astnode, arg):
        self.printline(astnode)
        self._inc_level()
        for param in astnode.params:
            param.visit(self)
            assert param.parent == astnode
        self._dec_level()

    def visit_CompareExpression(self, astnode, arg):
        self.printline(astnode)
        self._inc_level()
        astnode.expr1.visit(self)
        astnode.expr2.visit(self)
        assert astnode.expr1.parent == astnode
        assert astnode.expr2.parent == astnode
        self._dec_level()

    def visit_LiteralExpression(self, astnode, arg):
        self.printline(astnode)

    def visit_VariableExpression(self, astnode, arg):
        self.printline(astnode)
