"""
Command-line tool for making backups with FSL - not
tested enough to be of production quality!
"""

import datetime
import os
import shutil
from optparse import OptionParser

import fslparser
import HookHandler
import Interpreter
import utils

def _file_current_with(fileobj, other_filename):
    """Tell whether there exists file other_filename
    that is in sync with File instance fileobj."""
    if not os.path.exists(other_filename):
        return False
    if fileobj.get_size() != os.path.getsize(other_filename):
        return False
    modtime_sec = os.path.getmtime(other_filename) # Fix
    try:
        modtime = datetime.datetime.fromtimestamp(modtime_sec)
    except ValueError:
        return False
    if fileobj.get_modtime() != modtime:
        return False
    return True


class BackupHookHandler(HookHandler.HookHandler):
    def __init__(self, destdir, dryrun, stats_limit=1):
        self.destdir = destdir
        self.dryrun = dryrun
        self.copy_always = False
        self.stats_limit = stats_limit

        self.size_stack = []  # (dirname, files, bytes)
        self.total_copiedbytes = 0
        self.total_copiedfiles = 0
        
        self.errors = []

    def end(self):
        print
        print 'Total files copied: %s' % self.total_copiedfiles
        print 'Total bytes copied: %s' % utils.format_size(self.total_copiedbytes)
        
        if self.errors:
            print
            print '*** Errors ***'
            for msg in self.errors:
                print msg
                
        return len(self.errors) > 0

    def _calc_dest_filename(self, src_fileobj):
        absname =  src_fileobj.abs_filename.replace(':', '_')
        destname = os.path.join(self.destdir, absname)
        return os.path.normpath(destname)

    def _copyfile(self, src_fileobj, dest_filename):
        if (not self.copy_always) and _file_current_with(src_fileobj, dest_filename):
            return

        destdir = os.path.dirname(dest_filename)
        if not os.path.exists(destdir):
            os.makedirs(destdir)
        shutil.copy2(src_fileobj.abs_filename, dest_filename)
        self.total_copiedbytes += src_fileobj.get_size()
        self.total_copiedfiles += 1

    def include_file(self, fileobj, rule):
        dest_filename = self._calc_dest_filename(fileobj)
        if not self.dryrun:
            try:
                self._copyfile(fileobj, dest_filename)
            except (IOError, WindowsError), err:
                msg = '(%s) %s' % (fileobj.abs_filename, err)
                self.errors.append(msg)
                print '* Error *', msg

    def start_scan(self, dirname, glob_pattern):
        pass

    def end_scan(self, dirname, files_count, bytes_count):
        if bytes_count > self.stats_limit:
            print '%4d files, %8s %s' % (files_count, utils.format_size(bytes_count), dirname)


def main(argv):
    argv = filter(lambda a: a, argv) # Remove empty strings

    USAGE = 'fslbackup.py [options] <source-rootdir> <destination-dir> <rule-filenames>'
    parser = OptionParser(usage=USAGE)
    parser.add_option('-d', action='store_true', dest='dryrun',
        help="Dry run, don't copy anything")

    options, args = parser.parse_args(argv)

    if len(args) < 3:
        parser.print_usage()
        return
    rootdir = args[0]
    destdir = args[1]
    filenames = args[2:]

    ast = fslparser.parse(filenames, rootdir)

    if not os.path.exists(rootdir):
        print 'Root dir "%s" not found' % rootdir
        return
    if not os.path.isdir(rootdir):
        print 'Root dir "%s" is not a directory' % rootdir
        return

    if os.path.isfile(destdir):
        print 'Destination dir "%s" is a file' % destdir
        return

    if not os.path.exists(destdir) and not options.dryrun:
        os.makedirs(destdir)

    hook_handler = BackupHookHandler(destdir, options.dryrun, 10000)

    interpreter = Interpreter.Interpreter(ast)
    interpreter.add_hook_handler(hook_handler)
    interpreter.run()
    errors_occured = hook_handler.end()
    
    if errors_occured:
        sys.exit(1)
    else:
        sys.exit(0)

if __name__=='__main__':
    import sys
    main(sys.argv[1:])